/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef DXC_DATA_HPP
#define DXC_DATA_HPP

#include <iostream>
#include <string>

#include <DxcDefs.hpp>

using namespace std;

namespace Dxc
{
typedef long long Timestamp;

/**
 * @brief The root of the Dxc data definition hierarchy.
 */
class DXCAPI_EXPORT DxcData
{
public:
    /**
     * @brief Constructor
     *
     * @param timeStamp - The desired timestamp in milliseconds since 1970-1-1 00:00:00 GMT
     */
    DxcData(Timestamp timeStamp)
    {
        timeStamp_ = timeStamp;
    };

public:
    /**
     * @brief Get message timestamp
     *
     * @return The message timestamp in milliseconds since 1970-1-1 00:00:00 GMT
     */
    Timestamp getTimeStamp() const
    {
        return timeStamp_;
    };

    /**
     * @brief Set timestamp for DxcData
     *
     * Although timestamps can be freely manipulated by users in local instances
     * of DxcData, they serve only for informative purposes. There is no guarantee
     * that a DxcData message recipient won't overwrite the timestamp with the
     * time of message arrival.
     *
     * @param timeStamp - The desired timestamp in milliseconds since 1970-1-1 00:00:00 GMT
     */
    void setTimeStamp(Timestamp timeStamp)
    {
        timeStamp_ = timeStamp;
    };

    /**
     * @brief Virtual copy constructor.
     *
     * Returns a pointer to a copy of the DxcData. The user is
     * responsible for managing the memory.
     */
    virtual DxcData* clone() const = 0;

    /**
     * @brief Static method to obtain current time.
     *
     * @return The current time in milliseconds since 1970-1-1 00:00:00 GMT
     */
    static Timestamp currentTime();


    /**
     * @brief Prints DxcData in standardized, parseable format.
     */
    virtual ostream &put(ostream &) const = 0;

    virtual ~DxcData() {};
protected:
    DxcData();
    Timestamp timeStamp_;
};

}

ostream DXCAPI_EXPORT & operator << (ostream &, const Dxc::DxcData *);

#endif
