/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef DXC_SYSTEM_DESCRIPTION_HPP
#define DXC_SYSTEM_DESCRIPTION_HPP

#include <map>
#include <set>
#include <string>
#include <Parser/tinyxml.hpp>
#include <DxcDefs.hpp>

namespace Dxc
{


/**
 * @brief System Catalog parser.
 *
 * Provides a data structure for accessing information contained
 * in a system's XML catalog.
 */
class DXCAPI_EXPORT SystemCatalog
{
public:
    /**
     * @brief Get a system's catalog.
     *
     * @param system - the system's name
     *
     * System catalogs are searched for in $DXC_HOME/[SYSTEMCAT_DIR] (see DXC.cfg
     * for SYSTEMCAT_DIR).
     */
    static const SystemCatalog* getInstance(const std::string& system);

    /**
     * @brief The set of component IDs the system contains.
     */
    const std::set<std::string>* getComponentIDs() const;

    /**
     * @brief The set of mode IDs of a given component.
     *
     * @param componentID - component's ID
     */
    const std::set<std::string>* getFaultModeIDs(const std::string& componentID) const;

    /**
     * @brief Test the validity of a componentID.
     *
     * @param componentID - component's ID
     */
    bool isComponentValid(const std::string& componentID) const;

    /**
     * @brief Given a valid component, test the validity of a mode ID.
     *
     * @param componentID - component ID
     * @param modeID - mode ID
     */
    bool isFaultModeValid(const std::string& componentID, const std::string& modeID) const;

private:
    SystemCatalog(const std::string& system);
    TiXmlElement* findSiblingByValue(TiXmlElement* node,
                                     const std::string& elementName,
                                     const std::string& value);

    TiXmlElement* firstChild(TiXmlElement* node,
                             const std::string& elementName);

    static std::map<std::string, const SystemCatalog*>* myMap_;

    std::set<std::string> componentIDs_;
    std::map<std::string, std::set<std::string> > componentToModeIDs_;
};

}

#endif
