/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef VALUE_HPP
#define VALUE_HPP

#include <iostream>
#include <string>

#include <DxcDefs.hpp>

using namespace std;

namespace Dxc
{

/**
 * @brief Abstract interface for values that can be of various concrete datatypes.
 *
 * Value is the superclass of all the basic datatypes that can be referred to by
 * DxcData, such as boolean, integer, real, or string. The abstraction is provided
 * so that data of various kinds can be supplied in a uniform way, eg. for sensor
 * values and command values.
 */
class DXCAPI_EXPORT Value
{
public:
    Value();

public:
    /**
     * @brief Print the value to an output stream.
     */
    virtual ostream &put(ostream &) const = 0;
};

/**
 * @brief Boolean Value type.
 */
class DXCAPI_EXPORT BoolValue : public Value
{
public:
    BoolValue(bool);

public:
    bool get() const;
    virtual ostream &put(ostream &) const;

private:
    bool value_;
};

/**
 * @brief Real (double) Value type.
 */
class DXCAPI_EXPORT RealValue : public Value
{
public:
    double get() const;
    RealValue(double);

public:
    virtual ostream &put(ostream &) const;

private:
    double value_;
};

/**
 * @brief Integer (int) Value type.
 */
class DXCAPI_EXPORT IntegerValue : public Value
{
public:
    IntegerValue(int);

public:
    int get() const;
    virtual ostream &put(ostream &) const;

private:
    int value_;
};

/**
 * @brief String (STL std::string) Value type.
 */
class DXCAPI_EXPORT StringValue : public Value
{
public:
    StringValue(const string *);

public:
    string get() const;
    virtual ostream &put(ostream &) const;

private:
    const string *value_;
};

}

ostream DXCAPI_EXPORT & operator << (ostream &, const Dxc::Value *);

#endif
