/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.HashSet;

/**
 * Defines the data structure for a test article's Diagnosis.
 * 
 * A diagnosis is a set of candidates, weighed by their relative probability.
 * The weights of these candidates are normalized upon construction of the
 * datatype, resulting in weights that yield 1 when added together. An
 * additional detection signal and isolation signal indicate whether the
 * reasoner has detected something wrong with the system (detectionSignal =
 * true) and wether it has isolated the fault to specific component failure
 * modes (isolationSignal = true). If the isolationSignal is false, the
 * Candidate set, regardless of its contents, may be effectively treated as
 * empty.
 */
public class DiagnosisData extends DxcData {
    /**
     * Maps a set of component IDs to (hypothesized) faulty states.
     */
    public static class Candidate {
        public Candidate(Map<String, String> faultMap, double optionalWeight) {
            this.faultMap = faultMap;
            this.weight = optionalWeight;
        }

        public Candidate(Map<String, String> faultMap) {
            this.faultMap = faultMap;
            this.weight = -1;
        }

        public Candidate() {
            this.faultMap = new HashMap<String, String>();
            this.weight = -1;
        }

        public boolean equals(Object o) {
            Candidate c = (Candidate) o;
            if (c == null)
                return false;

            if (c.faultMap.equals(faultMap) && c.weight == weight)
                return true;
            return false;
        }

        public int hashCode() {
            int hash = 7;
            hash = 31 * hash + (int) (weight * 100);
            hash = 31 * hash + (faultMap == null ? 0 : faultMap.hashCode());
            return hash;
        }

        /**
         * A map of component IDs to predicted fault mode IDs.
         */
        public Map<String, String> faultMap;

        /**
         * Weight of this candidate relative to others in the diagnosis.
         * 
         *  Weights in a candidate set are normalized upon creation of
         *  DiagnosisData, such that they add up to 1.
         */
        public double weight;
    }

    private Set<Candidate> candidateSet;

    private boolean detectionSignal;

    private boolean isolationSignal;

    private String notes;

    /**
     * Constructor to initialize the DiagnosisData.
     * 
     * @param detectionSignal
     *            Set to true if the system is believed to be in a faulty state,
     *            even if there is not yet an isolation.
     * @param isolationSignal
     *            Set to true if faults have been isolated, ie. candidates
     *            exist.
     * @param candidateSet
     *            A set of Candidates (hypotheses). Their weights will be
     *            normalized, so that they all add up to one.
     * @param notes
     *            A string for diagnosis algorithms to supply additional
     *            information for their own purposes.
     */
    public DiagnosisData(boolean detectionSignal, boolean isolationSignal,
            Set<Candidate> candidateSet, String notes) {
        super();
        this.detectionSignal = detectionSignal;
        this.isolationSignal = isolationSignal;
        this.candidateSet = candidateSet;
        this.notes = (notes == null) ? "" : notes;

        if (this.candidateSet == null) {
            this.candidateSet = new HashSet<Candidate>();
        }

        normalizeWeights();
    }

    // this constructor is meant to be called from the parser
    public DiagnosisData(long timestamp, boolean detectionSignal,
            boolean isolationSignal, List<Map<String, String>> candidates,
            List<Double> weights, String notes) {
        super(timestamp);
        this.detectionSignal = detectionSignal;
        this.isolationSignal = isolationSignal;
        this.candidateSet = new HashSet<Candidate>();
        this.notes = (notes == null) ? "" : notes;

        if (candidates != null) {
            Iterator<Map<String, String>> ci = candidates.iterator();

            while (ci.hasNext()) {
                Candidate c = new Candidate(ci.next());

                if (!weights.isEmpty()) {
                    c.weight = weights.remove(0);
                }

                this.candidateSet.add(c);
            }

            normalizeWeights();
        }
    }

    /**
     * True if, according to the diagnosis, the system is believed to be in a
     * faulty state.
     */
    public boolean getDetectionSignal() {
        return detectionSignal;
    }

    /**
     * True if faults have been isolated, ie. candidates exist.
     */
    public boolean getIsolationSignal() {
        return isolationSignal;
    }

    /**
     * Returns test-article-specific notes.
     */
    public String getNotes() {
        return notes;
    }

    /**
     * Returns the set of candidates.
     */
    public java.util.Set<Candidate> getCandidateSet() {
        return candidateSet;
    }

    public PrintStream put(PrintStream out) {
        out.append("diagnosis @");
        out.append(Long.toString(this.getTimeStamp()));
        out.append(" detection = " + (this.detectionSignal ? "true" : "false")
                + ", ");
        out.append("isolation = " + (this.isolationSignal ? "true" : "false")
                + ", ");

        out.append("candidates = {");

        java.util.List<Double> weights = null;

        if (!candidateSet.isEmpty()) {
            weights = new java.util.LinkedList<Double>();

            java.util.Iterator<Candidate> i;
            i = candidateSet.iterator();
            while (i.hasNext()) {
                Candidate c = i.next();
                out.append("{ ");
                java.util.Iterator<String> j = c.faultMap.keySet().iterator();

                while (j.hasNext()) {
                    String s = j.next();
                    out.append(s + " = " + c.faultMap.get(s));
                    if (j.hasNext())
                        out.append(", ");
                }

                out.append("}");

                if (i.hasNext())
                    out.append(", ");
                weights.add(c.weight);
            }
        }

        out.append("}, weights = {");

        if (weights != null) {
            java.util.Iterator<Double> i;
            i = weights.iterator();
            while (i.hasNext()) {
                out.append(Double.toString((i.next())));

                if (i.hasNext())
                    out.append(", ");
            }
        }

        out.append("}, notes = '");
        out.append(notes + "';\n");

        return out;
    }

    private void normalizeWeights() {
        if (candidateSet.isEmpty())
            return;

        double total = 0;
        Iterator<Candidate> i = this.candidateSet.iterator();

        while (i.hasNext()) {
            Candidate c = i.next();
            if (c.weight < 0)
                c.weight = -c.weight;
            total += c.weight;
        }

        i = this.candidateSet.iterator();
        while (i.hasNext()) {
            Candidate c = i.next();
            c.weight /= total;
        }
    }
}
