/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import org.dxc.api.datatypes.Value;

/**
 * Defines the data structure for fault injection information message.
 * 
 * During an experiment, fault injection information is sent to the apropriate
 * parties to enable benchmarking of diagnosis algorithms. This information will
 * be inaccessible by diagnosers during competition scenarios. Unlike
 * DiagnosisData, FaultInjectData is used to communicate an *actual* faulty
 * component and mode.
 */
public class FaultInjectData extends DxcData {
    private String componentID;

    private String modeID;

    private boolean isInjection;

    private java.util.Map<String, Value> parameters;

    /**
     * Constructor.
     * 
     * @param timestamp
     *            - message time.
     * @param isInjection
     *            - was the fault injected (true), or removed (false)?
     * @param componentID
     *            - the component that has entered/exited a faulty state.
     * @param modeID
     *            - the name of the mode the component is entering/exiting.
     * @param parameters
     *            - internal fault parameters.
     */
    public FaultInjectData(Long timestamp, String componentID, String modeID,
            boolean isInjection, Map<String, Value> parameters) {
        super(timestamp);
        this.componentID = (componentID == null) ? "" : componentID;
        this.modeID = (modeID == null) ? "" : modeID;
        this.isInjection = isInjection;
        this.parameters = (parameters == null) ? new HashMap<String, Value>()
                : parameters;
    }

    /**
     * Returns the name of the specific component the fault is being injected
     * into.
     * 
     * A fault's component ID together with its failure mode serve to uniquely
     * identify a fault.
     */
    public String getComponentID() {
        return componentID;
    }

    /**
     * Fault mode.
     * 
     * The mode ID determines the nature of the fault being injected. A fault's
     * component ID together with its mode ID serve to uniquely identify a
     * fault.
     */
    public String getModeID() {
        if (modeID != null && !modeID.equals(""))
            return modeID;
        else
            return "No Fault";
    }

    /**
     * True if the fault in question is being injected, false if it's being
     * stopped.
     */
    public boolean isInjection() {
        return isInjection;
    }

    /**
     * Set to true if the fault is being injected, false if it's being stopped.
     */
    public void isInjection(boolean isInjection) {
        this.isInjection = isInjection;
    }

    /**
     * Internal parameter map.
     */
    public java.util.Map<String, Value> getParameterMap() {
        return parameters;
    }

    public PrintStream put(PrintStream out) {

        out.append("faultInjection @");
        out.append(Long.toString(this.getTimeStamp()));
        if (isInjection)
            out.append(" isInjection = true, ");
        else
            out.append(" isInjection = false, ");

        out.append("fault = {" + componentID + " = ");
        out.append(modeID + "}, ");

        out.append("parameters = {");

        if (!parameters.isEmpty()) {
            java.util.Iterator<String> i = parameters.keySet().iterator();

            while (i.hasNext()) {
                String s = i.next();
                out.append(s + " = ");
                parameters.get(s).put(out);

                if (i.hasNext())
                    out.append(", ");
            }
        }

        out.append("};\n");

        return out;
    }
}
