/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
/**
 * ExampleDA demonstrates how a diagnosis algorithm should
 * send and receive scenario data.
 */

#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#endif

#include <iostream>

#include <DxcCallback.hpp>
#include <DxcDataTypes.hpp>
#include <Connection/ConnectorFactory.hpp>

using namespace Dxc;
using namespace std;

static DxcConnector *connector;
static bool run = true;

class ExampleDA
{
public:
    void receivedSensorData();
    void receivedCommandData();
};

void ExampleDA::receivedSensorData()
{
    msleep(50);

    // In this example we send a diagnosis for every received SensorData.
    DiagnosisData::CandidateSet exampleCandidateSet;
    DiagnosisData::Candidate c1, c2, c3;

    /* Here we populate candidates with made-up data.
     * In reality component IDs and mode IDs must match
     * XML system descriptions.
     */
    c1.faultMap["c"] = "stuckOpen";
    c1.faultMap["d"] = "stuckClosed";
    c1.weight = 0.2;

    c2.faultMap["a"] = "bias";
    c2.faultMap["b"] = "switchedOff";
    c2.weight = 0.3;

    c3.faultMap["c"] = "stuckClosed";
    c3.faultMap["d"] = "stuckOpen";
    c3.weight = 0.5;

    exampleCandidateSet.insert(c1);
    exampleCandidateSet.insert(c2);
    exampleCandidateSet.insert(c3);

    DiagnosisData* exampleDiagnosis = new DiagnosisData(true, true,
            exampleCandidateSet, "example diagnosis");

    connector->sendMessage(exampleDiagnosis);
}

void ExampleDA::receivedCommandData()
{
}

ExampleDA *exampleDA = NULL;

/**
 * This is the DA's implementation of the DxcCallback interface.
 * After an instance of this class is registered with a DxcConnector,
 * the connector will execute its processData() method every time
 * data is received.
 */
class ExampleCallback : public DxcCallback
{
private:
    virtual void processData(const DxcData * msg)
    {
        /* msg is incoming data. Find out what type it is,
         * downcast it to that DxcData type, and process accordingly.
         */
        if (string(typeid(*msg).name()) == string(typeid(SensorData).name()))
        {
            // update observations
            const SensorData* sd = dynamic_cast<const SensorData*>(msg);
            SensorData::SensorValueMap sensors = sd->getSensorValueMap();
            SensorData::SensorValueMap::iterator i = sensors.begin();

            for (; i != sensors.end(); i++)
            {
                string sensorID = i->first;
                const Value* v = i->second;

                /* Here we don't know what datatype the value is, but this
                 * information can be looked up by sensor ID in the system's
                 * XML catalog (TinyXML, a lightweight parser, is included
                 * in <Parser/tinyxml.hpp>)
                 */
                if (string(typeid(*v).name()) ==
                        string(typeid(StringValue).name()))
                {
                    string sv = dynamic_cast<const StringValue*>(v)->get();
                    // write (sensorID, sv) to DA model/data
                    // structure for diagnosis
                }
                else if (string(typeid(*v).name()) ==
                         string(typeid(BoolValue).name()))
                {
                    bool bv = dynamic_cast<const BoolValue*>(v)->get();
                    // write (sensorID, bv) to DA model/data
                    // structure for diagnosis
                }
                else if (string(typeid(*v).name()) ==
                         string(typeid(IntegerValue).name()))
                {
                    int iv = dynamic_cast<const IntegerValue*>(v)->get();
                    // write (sensorID, iv) to DA model/data
                    // structure for diagnosis
                }
                else if (string(typeid(*v).name()) == string(typeid(RealValue).name()))
                {
                    double rv = dynamic_cast<const RealValue*>(v)->get();
                    // write (sensorID, rv) to DA model/data
                    // structure for diagnosis
                }
            }

            exampleDA->receivedSensorData();
        }
        else if (string(typeid(*msg).name()) ==
                 string(typeid(CommandData).name()))
        {
            const CommandData* cd = dynamic_cast<const CommandData*>(msg);
            string commandID = cd->getCommandID();
            const Value* v = cd->getCommandValue();

            /* Here we don't know what datatype the value is, but this
             * information can be looked up by command ID in the system's
             * XML catalog (TinyXML, a lightweight parser, is included
             * in <Parser/tinyxml.hpp>)
             */
            if (string(typeid(*v).name()) ==
                    string(typeid(StringValue).name()))
            {
                string sv = dynamic_cast<const StringValue*>(v)->get();
                // write (commandID, sv) to DA model/data
                // structure for diagnosis
            }
            else if (string(typeid(*v).name()) ==
                     string(typeid(BoolValue).name()))
            {
                bool bv = dynamic_cast<const BoolValue*>(v)->get();
                // write (commandID, bv) to DA model/data
                // structure for diagnosis
            }
            else if (string(typeid(*v).name()) ==
                     string(typeid(IntegerValue).name()))
            {
                int iv = dynamic_cast<const IntegerValue*>(v)->get();
                // write (commandID, iv) to DA model/data
                // structure for diagnosis
            }
            else if (string(typeid(*v).name()) ==
                     string(typeid(RealValue).name()))
            {
                double rv = dynamic_cast<const RealValue*>(v)->get();
                // write (commandID, rv) to DA model/data
                // structure for diagnosis
            }

            exampleDA->receivedCommandData();
        }
        else if (string(typeid(*msg).name()) ==
                 string(typeid(ScenarioStatusData).name()))
        {
            // check for scenario end, exit if ended
            const ScenarioStatusData* stat =
                dynamic_cast<const ScenarioStatusData *>(msg);

            if (stat->getStatus() == ScenarioStatusData::SDS_ENDED)
            {
                run = false;
            }
        }
        else if (typeid(msg).name() == typeid(ErrorData).name())
        {
            cout << "DA received error: " << msg << endl;
        }
    };
};

int main(int argc, char **argv)
{
    /* Create our implementation of DxcCallback and pass it to a new
     * DxcConnector. A scenario data source (SDS) must be running
     * in order to connect successfully.
     */
    ExampleCallback callback;
    connector = ConnectorFactory::getDAConnector(&callback);

    exampleDA = new ExampleDA;

    // signal to the system that we're prepared to receive data
    ScenarioStatusData ssd(ScenarioStatusData::DA_READY);
    connector->sendMessage(&ssd);

    // Sleep now. All further computations are done within processData().
    while (run)
    {
        msleep(100);
    }

    delete exampleDA;

    // clean up after yourself!
    delete connector;

    return 0;
}
