package org.dxc.evaluator;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dxc.api.datatypes.DiagnosisData;
import org.dxc.api.datatypes.FaultInjectData;
import org.dxc.api.datatypes.SystemCatalog;

public class DAResult {
    private String name;
    private String system;
    private int scenarioCount;
    private List<Long> detectionTimes;
    private List<Long> isolationTimes;
    private List<Long> cpuTimes;
    private List<Long> memUsages;

    // dimension 1 is the actual state (0 = nominal, 1 = faulty),
    // dimension 2 is the diagnosed state
    private int[][] detectionMatrix;
    private Map<String, ConfusionMatrix> idToConfMat;

    private static boolean printStats = true;

    DAResult(String DAName, String system) throws Exception {
        this.name = DAName;
        this.system = system;
        this.scenarioCount = 0;

        detectionMatrix = new int[2][2];
        idToConfMat = new HashMap<String, ConfusionMatrix>();
        detectionTimes = new LinkedList<Long>();
        isolationTimes = new LinkedList<Long>();
        cpuTimes = new LinkedList<Long>();
        memUsages = new LinkedList<Long>();

        loadConfusionMatrices();
    }

    public String getDAName() {
        return name;
    }

    public String getSystem() {
        return system;
    }

    public void addScenario(ScenarioResult sr) {
        if (printStats)
            System.out.println("Scenario: " + sr.getName());
        List<DiagnosisData> dds = sr.getDiagnoses();
        List<FaultInjectData> fids = sr.getFaultInjections();

        updateDetection(fids, dds);
        updateIsolation(fids, dds);
        cpuTimes.add(sr.getCpuTime());
        memUsages.add(sr.getMaxMem());
        scenarioCount++;

        if (printStats)
            System.out.println();
    }

    public double getMeanTimeToDetection() {
        return averageL(detectionTimes);
    }

    public double getMeanTimeToIsolation() {
        return averageL(isolationTimes);
    }

    public double getFalsePositivesRate() {
        double totalNominal = detectionMatrix[0][0] + detectionMatrix[0][1];
        if (totalNominal == 0)
            return 0;
        return detectionMatrix[0][1] / (double) (totalNominal);
    }

    public double getFalseNegativesRate() {
        double totalFaulty = detectionMatrix[1][0] + detectionMatrix[1][1];
        if (totalFaulty == 0)
            return 0;
        return detectionMatrix[1][0] / (double) (totalFaulty);
    }

    public double getDetectionAccuracy() {
        return (detectionMatrix[0][0] + detectionMatrix[1][1])
                / (double) scenarioCount;
    }

    public double getIsolationAccuracy() {
        List<Double> compRate = new LinkedList<Double>();
        Iterator<ConfusionMatrix> ci = idToConfMat.values().iterator();

        while (ci.hasNext()) {
            ConfusionMatrix c = ci.next();
            compRate.add(c.percentCorrect());
        }
        return averageD(compRate);
    }

    public double getMeanCpuTime() {
        return averageL(cpuTimes);
    }

    public double getMeanMemUsage() {
        return averageL(memUsages);
    }

    private void loadConfusionMatrices() throws Exception {
        SystemCatalog sc = SystemCatalog.getInstance(system);

        Set<String> componentIDs = sc.getComponentIDs();
        Iterator<String> i = componentIDs.iterator();

        while (i.hasNext()) {
            String id = i.next();
            Set<String> modes = sc.getFaultModes(id);
            modes.add(ConfusionMatrix.NOMINAL);

            idToConfMat.put(id, new ConfusionMatrix(modes));
        }
    }

    private void updateDetection(List<FaultInjectData> fids,
            List<DiagnosisData> dds) {
        long injectionTime = -1, detectionTime = -1;

        if (!fids.isEmpty()) {
            // assumes all faultInject messages are injections
            injectionTime = fids.get(0).getTimeStamp();
        }

        Iterator<DiagnosisData> ddi = dds.iterator();
        boolean detection = false;

        while (ddi.hasNext()) {
            DiagnosisData diag = ddi.next();
            if (diag.getDetectionSignal()) {
                detectionTime = diag.getTimeStamp();
                detection = true;
                break;
            }
        }
        /*
         * uncomment to take detection signal from _last_ diagnosis if
         * (!dds.isEmpty()) detection =
         * dds.get(dds.size()-1).getDetectionSignal();
         */
        if (printStats)
            System.out.print("Detection: ");

        if (detection) {
            if (injectionTime == -1 // <- no faults.
                    || detectionTime < injectionTime) { // <- detected early.
                detectionMatrix[0][1]++; // false positive
                if (printStats)
                    System.out.println("False positive.");
            } else {
                detectionMatrix[1][1]++; // true positive
                detectionTimes.add(detectionTime - injectionTime);
                if (printStats) {
                    System.out.print("True positive. Detection time: ");
                    System.out.println(detectionTime - injectionTime);
                }
            }
        } else {
            if (injectionTime == -1) {
                detectionMatrix[0][0]++; // true negative
                if (printStats)
                    System.out.println("True negative.");
            } else {
                detectionMatrix[1][0]++; // false negative
                if (printStats)
                    System.out.println("False negative.");
            }
        }

    }

    private void updateIsolation(List<FaultInjectData> fids,
            List<DiagnosisData> dds) {
        Map<String, String> faultMap = new HashMap<String, String>();
        DiagnosisData isolation = null;
        long injectionTime = -1, isolationTime = -1;

        if (!fids.isEmpty()) {
            injectionTime = fids.get(0).getTimeStamp();
        }

        Iterator<FaultInjectData> i = fids.iterator();
        while (i.hasNext()) {
            FaultInjectData f = i.next();
            faultMap.put(f.getComponentID(), f.getModeID());
        }

        if (!dds.isEmpty()) {
            for (int j = dds.size() - 1; j >= 0; j--) {
                isolation = dds.get(j);
                isolationTime = isolation.getTimeStamp();
                if (j > 0) {
                    Set<DiagnosisData.Candidate> s1 = isolation
                            .getCandidateSet(), s2 = dds.get(j - 1)
                            .getCandidateSet();
                    if (!s1.equals(s2))
                        break;
                }
            }
        } else {
            isolation = new DiagnosisData(false, false, null, null);
        }

        if (isolationTime > 0) {
            if (injectionTime > 0 && injectionTime < isolationTime) {
                this.isolationTimes.add(isolationTime - injectionTime);
                if (printStats)
                    System.out.println("Isolation time: "
                            + (isolationTime - injectionTime));
            }
        }

        Iterator<DiagnosisData.Candidate> cs = isolation.getCandidateSet()
                .iterator();
        while (cs.hasNext()) {
            DiagnosisData.Candidate c = cs.next();
            Iterator<String> si = c.faultMap.keySet().iterator();
            while (si.hasNext()) {
                String id = si.next();
                if (!idToConfMat.containsKey(id)) {
                    System.out.println("Unknown component ID in diagnosis: "
                            + id);
                }
            }
        }

        Iterator<String> ids = idToConfMat.keySet().iterator();
        while (ids.hasNext()) {
            String id = ids.next();
            String actual = ConfusionMatrix.NOMINAL;
            Map<String, Double> predictions = new HashMap<String, Double>();

            predictions.put(ConfusionMatrix.NOMINAL, 1.0);

            if (faultMap.containsKey(id)) {
                actual = faultMap.get(id);
            }

            if (isolation.getCandidateSet() != null) {
                Iterator<DiagnosisData.Candidate> ci = isolation
                        .getCandidateSet().iterator();

                while (ci.hasNext()) {
                    DiagnosisData.Candidate c = ci.next();

                    if (c.faultMap.containsKey(id)) {
                        String mode = c.faultMap.get(id);
                        Double weight = predictions.get(mode);
                        if (weight == null) {
                            weight = c.weight;
                        } else {
                            weight += c.weight;
                        }
                        predictions.put(mode, weight);

                        Double nomWeight = predictions
                                .get(ConfusionMatrix.NOMINAL);
                        nomWeight -= c.weight;
                        predictions.put(ConfusionMatrix.NOMINAL, nomWeight);

                        predictions.put(c.faultMap.get(id), weight);
                    }
                }
            }

            Iterator<String> j = predictions.keySet().iterator();
            while (j.hasNext()) {
                String prediction = j.next();
                Double weight = predictions.get(prediction);
                idToConfMat.get(id).addPrediction(actual, prediction, weight);
            }
        }

    }

    private double averageD(List<Double> pts) {
        Iterator<Double> i = pts.iterator();
        Double total = new Double(0);

        while (i.hasNext()) {
            Double n = i.next();
            total += n;
        }
        return total / pts.size();
    }

    private double averageL(List<Long> pts) {
        Iterator<Long> i = pts.iterator();
        Double total = new Double(0);

        while (i.hasNext()) {
            Long n = i.next();
            total += n;
        }
        return total / pts.size();
    }
}
