/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.standaloneSDS;

import org.dxc.api.DxcCallback;
import org.dxc.api.connection.ConnectorFactory;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.datatypes.*;
import org.dxc.api.parser.DxcParser;

import java.io.FileReader;
import java.io.PrintStream;

public class StandaloneSDS {
    private static PrintStream output;

    private static final DxcCallback writeToFileCallback = new DxcCallback() {
        public void processData(DxcData data) {
            if (data instanceof ScenarioStatusData) {
                if (((ScenarioStatusData) data).getStatus().equals(
                        ScenarioStatusData.DA_READY)) {
                    notifyAll();
                }
            }
            if (output != null) {
                data.put(output);
            }
        }
    };

    private static void printUsage() {
        System.out.print("Usage: ");
        System.out.println(StandaloneSDS.class.getCanonicalName());
        System.out.println("                         "
                + "[-h ][-t waitTime] input_file [output_file]");
        System.out.println();
        System.out.println("       [-h]:             "
                + "display this message.");
        System.out.println("       [-t wait_time]:   "
                + "specifies how long (seconds) StandaloneSDS ");
        System.out.println("                         "
                + "should wait after scenario end before exiting.");
        System.out.println("       input_file:       "
                + "scenario to run (must be in proper format).");
        System.out.println("       [output_file]:    "
                + "optional file to store results (otherwise");
        System.out.println("                         use stdout).");
        System.out.println();
    }

    public static void main(String[] args) {
        try {
            long waitTime = 0;
            if (args.length < 1) {
                printUsage();
            } else {
                int pos = 0;

                while (pos < args.length && args[pos].startsWith("-")) {
                    if (args[pos].equals("-h")) {
                        printUsage();
                        pos++;
                    } else if (args[pos].equals("-t")) {
                        try {
                            waitTime = Long.parseLong(args[++pos]);
                            pos++;
                        } catch (Exception ex) {
                            throw new Exception("-t: couldn't parse wait time.");
                        }
                    } else {
                        throw new Exception(args[pos] + ": unknown option.");
                    }
                }

                System.out.print("\n");

                if (pos >= args.length) {
                    throw new Exception("No input file specified.");
                }

                FileReader input = new FileReader(args[pos++]);

                if (pos < args.length) {
                    System.out.println("Writing output to: " + args[pos]);
                    output = new PrintStream(args[pos]);
                } else {
                    System.out.println("Writing output to stdout.");
                    output = System.out;
                }

                DxcConnector c = ConnectorFactory
                        .getStandaloneSDSConnector(writeToFileCallback);
                ForwardMessageCallback fmc = new ForwardMessageCallback(c);

                synchronized (writeToFileCallback) {
                    System.out.println("Awaiting DA...");
                    writeToFileCallback.wait();
                }

                System.out.println("Starting.");
                c
                        .sendMessage(new ScenarioStatusData(
                                ScenarioStatusData.STARTED));

                DxcParser parser = new DxcParser(fmc, input);
                parser.beginParse();

                c.sendMessage(new ScenarioStatusData(ScenarioStatusData.ENDED));
                Thread.sleep(waitTime * 1000);
            }
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }

        System.exit(0);
    }

}
